@@ -14,12 +14,15 @@
 
 """Exceptions for the Brick library."""
 
-from oslo_concurrency import processutils as putils
-import six
+from __future__ import annotations
+
 import traceback
+from typing import Any, Optional  # noqa: H301
+
+from oslo_concurrency import processutils as putils
+from oslo_log import log as logging
 
 from os_brick.i18n import _
-from oslo_log import log as logging
 
 
 LOG = logging.getLogger(__name__)
@@ -29,12 +32,12 @@ class BrickException(Exception):
     """Base Brick Exception
 
     To correctly use this class, inherit from it and define
-    a 'msg_fmt' property. That msg_fmt will get printf'd
+    a 'message' property. That message will get printf'd
     with the keyword arguments provided to the constructor.
     """
     message = _("An unknown exception occurred.")
     code = 500
-    headers = {}
+    headers: dict = {}
     safe = False
 
     def __init__(self, message=None, **kwargs):
@@ -67,9 +70,6 @@ class BrickException(Exception):
         self.msg = message
         super(BrickException, self).__init__(message)
 
-    def __unicode__(self):
-        return six.text_type(self.msg)
-
 
 class NotFound(BrickException):
     message = _("Resource could not be found.")
@@ -120,7 +120,7 @@ class TargetPortalNotFound(BrickException):
     message = _("Unable to find target portal %(target_portal)s.")
 
 
-class TargetPortalsNotFound(BrickException):
+class TargetPortalsNotFound(TargetPortalNotFound):
     message = _("Unable to find target portal in %(target_portals)s.")
 
 
@@ -186,8 +186,9 @@ class ExceptionChainer(BrickException):
     logged with warning level.
     """
     def __init__(self, *args, **kwargs):
-        self._exceptions = []
-        self._repr = None
+        self._exceptions: list[tuple] = []
+        self._repr: Optional[str] = None
+        self._exc_msg_args = []
         super(ExceptionChainer, self).__init__(*args, **kwargs)
 
     def __repr__(self):
@@ -200,23 +201,26 @@ class ExceptionChainer(BrickException):
                                    for i, t in enumerate(tracebacks))
         return self._repr
 
-    __str__ = __unicode__ = __repr__
+    __str__ = __repr__
 
-    def __nonzero__(self):
+    def __nonzero__(self) -> bool:
         # We want to be able to do boolean checks on the exception
         return bool(self._exceptions)
 
     __bool__ = __nonzero__  # For Python 3
 
-    def add_exception(self, exc_type, exc_val, exc_tb):
+    def add_exception(self, exc_type, exc_val, exc_tb) -> None:
         # Clear the representation cache
         self._repr = None
         self._exceptions.append((exc_type, exc_val, exc_tb))
 
-    def context(self, catch_exception, msg='', *msg_args):
+    def context(self,
+                catch_exception: bool,
+                msg: str = '',
+                *msg_args: Any):
         self._catch_exception = catch_exception
         self._exc_msg = msg
-        self._exc_msg_args = msg_args
+        self._exc_msg_args = list(msg_args)
         return self
 
     def __enter__(self):
