@@ -14,12 +14,10 @@
 
 import json
 import os
-import requests
-import six
-from six.moves import urllib
+import urllib
 
-from oslo_concurrency import lockutils
 from oslo_log import log as logging
+import requests
 
 from os_brick import exception
 from os_brick.i18n import _
@@ -31,7 +29,6 @@ from os_brick import utils
 LOG = logging.getLogger(__name__)
 DEVICE_SCAN_ATTEMPTS_DEFAULT = 3
 CONNECTOR_CONF_PATH = '/opt/emc/scaleio/openstack/connector.conf'
-synchronized = lockutils.synchronized_with_prefix('os-brick-')
 
 
 def io(_type, nr):
@@ -76,6 +73,8 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         self.volume_path = None
         self.iops_limit = None
         self.bandwidth_limit = None
+        self.verify_certificate = None
+        self.certificate_path = None
 
     def _get_guid(self):
         try:
@@ -100,7 +99,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             password = priv_scaleio.get_connector_password(
                 CONNECTOR_CONF_PATH,
                 connection_properties['config_group'],
-                connection_properties['failed_over'])
+                connection_properties.get('failed_over', False))
             return password, None
         except Exception as e:
             msg = _("Error getting ScaleIO connector password from "
@@ -154,7 +153,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
     # NOTE: Usually 3 retries is enough to find the volume.
     # If there are network issues, it could take much longer. Set
     # the max retries to 15 to make sure we can find the volume.
-    @utils.retry(exceptions=exception.BrickException,
+    @utils.retry(exception.BrickException,
                  retries=15,
                  backoff_rate=1)
     def _wait_for_volume_path(self, path):
@@ -205,7 +204,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         r = requests.get(
             request,
             auth=(self.server_username, self.server_token),
-            verify=False
+            verify=self._verify_cert()
         )
 
         r = self._check_response(r, request)
@@ -251,7 +250,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
 
         r = requests.get(request,
                          auth=(self.server_username, self.server_token),
-                         verify=False)
+                         verify=self._verify_cert())
 
         r = self._check_response(r, request)
 
@@ -291,7 +290,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             r = requests.get(
                 login_request,
                 auth=(self.server_username, self.server_password),
-                verify=False
+                verify=self._verify_cert()
             )
 
             token = r.json()
@@ -302,7 +301,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             if is_get_request:
                 res = requests.get(request,
                                    auth=(self.server_username, token),
-                                   verify=False)
+                                   verify=self._verify_cert())
             else:
                 headers = {'content-type': 'application/json'}
                 res = requests.post(
@@ -310,7 +309,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
                     data=json.dumps(params),
                     headers=headers,
                     auth=(self.server_username, token),
-                    verify=False
+                    verify=self._verify_cert()
                 )
 
             self.server_token = token
@@ -318,6 +317,12 @@ class ScaleIOConnector(base.BaseLinuxConnector):
 
         return response
 
+    def _verify_cert(self):
+        verify_cert = self.verify_certificate
+        if self.verify_certificate and self.certificate_path:
+            verify_cert = self.certificate_path
+        return verify_cert
+
     def get_config(self, connection_properties):
         self.local_sdc_ip = connection_properties['hostIP']
         self.volume_name = connection_properties['scaleIO_volname']
@@ -332,12 +337,17 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             connection_properties)
         self.iops_limit = connection_properties['iopsLimit']
         self.bandwidth_limit = connection_properties['bandwidthLimit']
+        self.verify_certificate = (
+            connection_properties.get('verify_certificate')
+        )
+        self.certificate_path = connection_properties.get('certificate_path')
         device_info = {'type': 'block',
                        'path': self.volume_path}
         return device_info
 
     @utils.trace
-    @lockutils.synchronized('scaleio', 'scaleio-')
+    @utils.connect_volume_prepare_result
+    @base.synchronized('scaleio', 'scaleio-', external=True)
     def connect_volume(self, connection_properties):
         """Connect the volume.
 
@@ -383,7 +393,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             data=json.dumps(params),
             headers=headers,
             auth=(self.server_username, self.server_token),
-            verify=False
+            verify=self._verify_cert()
         )
 
         r = self._check_response(r, request, False, params)
@@ -432,7 +442,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
                 data=json.dumps(params),
                 headers=headers,
                 auth=(self.server_username, self.server_token),
-                verify=False
+                verify=self._verify_cert()
             )
             r = self._check_response(r, request, False, params)
             if r.status_code != self.OK_STATUS_CODE:
@@ -451,7 +461,8 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         return device_info
 
     @utils.trace
-    @lockutils.synchronized('scaleio', 'scaleio-')
+    @base.synchronized('scaleio', 'scaleio-', external=True)
+    @utils.connect_volume_undo_prepare_result(unlink_after=True)
     def disconnect_volume(self, connection_properties, device_info,
                           force=False, ignore_errors=False):
         """Disconnect the ScaleIO volume.
@@ -496,7 +507,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             data=json.dumps(params),
             headers=headers,
             auth=(self.server_username, self.server_token),
-            verify=False
+            verify=self._verify_cert()
         )
 
         r = self._check_response(r, request, False, params)
@@ -515,6 +526,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
                 LOG.error(msg)
                 raise exception.BrickException(message=msg)
 
+    @utils.connect_volume_undo_prepare_result
     def extend_volume(self, connection_properties):
         """Update the local kernel's size information.
 
@@ -525,22 +537,9 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         self._rescan_vols()
         volume_paths = self.get_volume_paths(connection_properties)
         if volume_paths:
-            return self.get_device_size(volume_paths[0])
+            return utils.get_device_size(self, volume_paths[0])
 
         # if we got here, the volume is not mapped
         msg = (_("Error extending ScaleIO volume"))
         LOG.error(msg)
         raise exception.BrickException(message=msg)
-
-    def get_device_size(self, device):
-        """Get the size in bytes of a volume."""
-        (out, _err) = self._execute('blockdev', '--getsize64',
-                                    device, run_as_root=True,
-                                    root_helper=self._root_helper)
-        var = six.text_type(out.strip())
-        LOG.debug("Device %(dev)s size: %(var)s",
-                  {'dev': device, 'var': var})
-        if var.isnumeric():
-            return int(var)
-        else:
-            return None
