@@ -16,14 +16,18 @@
 
    Note, this is not iSCSI.
 """
+
+from __future__ import annotations
+
 import glob
 import os
 import re
 import time
+from typing import Optional
 
 from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
-import six
+from oslo_utils import excutils
 
 from os_brick import exception
 from os_brick import executor
@@ -37,13 +41,14 @@ MULTIPATH_WWID_REGEX = re.compile(r"\((?P<wwid>.+)\)")
 MULTIPATH_DEVICE_ACTIONS = ['unchanged:', 'reject:', 'reload:',
                             'switchpg:', 'rename:', 'create:',
                             'resize:']
+MULTIPATHD_RESIZE_TIMEOUT = 120
 
 
 class LinuxSCSI(executor.Executor):
     # As found in drivers/scsi/scsi_lib.c
     WWN_TYPES = {'t10.': '1', 'eui.': '2', 'naa.': '3'}
 
-    def echo_scsi_command(self, path, content):
+    def echo_scsi_command(self, path, content) -> None:
         """Used to echo strings to scsi subsystem."""
 
         args = ["-a", path]
@@ -52,7 +57,7 @@ class LinuxSCSI(executor.Executor):
                       root_helper=self._root_helper)
         self._execute('tee', *args, **kwargs)
 
-    def get_name_from_path(self, path):
+    def get_name_from_path(self, path) -> Optional[str]:
         """Translates /dev/disk/by-path/ entry to /dev/sdX."""
 
         name = os.path.realpath(path)
@@ -61,8 +66,11 @@ class LinuxSCSI(executor.Executor):
         else:
             return None
 
-    def remove_scsi_device(self, device, force=False, exc=None,
-                           flush=True):
+    def remove_scsi_device(self,
+                           device: str,
+                           force: bool = False,
+                           exc=None,
+                           flush: bool = True) -> None:
         """Removes a scsi device based upon /dev/sdX name."""
         path = "/sys/block/%s/device/delete" % device.replace("/dev/", "")
         if os.path.exists(path):
@@ -77,7 +85,7 @@ class LinuxSCSI(executor.Executor):
             with exc.context(force, 'Removing %s failed', device):
                 self.echo_scsi_command(path, "1")
 
-    def wait_for_volumes_removal(self, volumes_names):
+    def wait_for_volumes_removal(self, volumes_names: list[str]) -> None:
         """Wait for device paths to be removed from the system."""
         str_names = ', '.join(volumes_names)
         LOG.debug('Checking to see if SCSI volumes %s have been removed.',
@@ -100,7 +108,7 @@ class LinuxSCSI(executor.Executor):
                     LOG.debug('%s still exist.', ', '.join(exist))
         raise exception.VolumePathNotRemoved(volume_path=exist)
 
-    def get_device_info(self, device):
+    def get_device_info(self, device: str) -> dict[str, Optional[str]]:
         dev_info = {'device': device, 'host': None,
                     'channel': None, 'id': None, 'lun': None}
         # The input argument 'device' can be of 2 types:
@@ -127,7 +135,7 @@ class LinuxSCSI(executor.Executor):
         LOG.debug('dev_info=%s', str(dev_info))
         return dev_info
 
-    def get_sysfs_wwn(self, device_names, mpath=None):
+    def get_sysfs_wwn(self, device_names: list[str], mpath=None) -> str:
         """Return the wwid from sysfs in any of devices in udev format."""
         # If we have a multipath DM we know that it has found the WWN
         if mpath:
@@ -149,7 +157,7 @@ class LinuxSCSI(executor.Executor):
             return wwid
 
         # If we have multiple designators use symlinks to find out the wwn
-        device_names = set(device_names)
+        device_names_set = set(device_names)
         for wwn_path in wwn_paths:
             try:
                 if os.path.islink(wwn_path) and os.stat(wwn_path):
@@ -165,11 +173,11 @@ class LinuxSCSI(executor.Executor):
                             dm_devs = os.listdir(slaves_path)
                             # This is the right wwn_path if the devices we have
                             # attached belong to the dm we followed
-                            if device_names.intersection(dm_devs):
+                            if device_names_set.intersection(dm_devs):
                                 break
 
                         # This is the right wwn_path if  devices we have
-                        elif name in device_names:
+                        elif name in device_names_set:
                             break
             except OSError:
                 continue
@@ -192,7 +200,7 @@ class LinuxSCSI(executor.Executor):
             return udev_wwid
         return ''
 
-    def get_scsi_wwn(self, path):
+    def get_scsi_wwn(self, path: str) -> str:
         """Read the WWN from page 0x83 value for a SCSI device."""
 
         (out, _err) = self._execute('/lib/udev/scsi_id', '--page', '0x83',
@@ -202,7 +210,9 @@ class LinuxSCSI(executor.Executor):
         return out.strip()
 
     @staticmethod
-    def is_multipath_running(enforce_multipath, root_helper, execute=None):
+    def is_multipath_running(enforce_multipath,
+                             root_helper,
+                             execute=None) -> bool:
         try:
             if execute is None:
                 execute = priv_rootwrap.execute
@@ -213,13 +223,13 @@ class LinuxSCSI(executor.Executor):
             # code and just printed the error message in stdout.
             if out and out.startswith('error receiving packet'):
                 raise putils.ProcessExecutionError('', out, 1, cmd, None)
+
         except putils.ProcessExecutionError as err:
-            LOG.error('multipathd is not running: exit code %(err)s',
-                      {'err': err.exit_code})
             if enforce_multipath:
+                LOG.error('multipathd is not running: exit code %(err)s',
+                          {'err': err.exit_code})
                 raise
             return False
-
         return True
 
     def get_dm_name(self, dm):
@@ -249,14 +259,6 @@ class LinuxSCSI(executor.Executor):
                 return dm
         return None
 
-    @staticmethod
-    def get_dev_path(connection_properties, device_info):
-        """Determine what path was used by Nova/Cinder to access volume."""
-        if device_info and device_info.get('path'):
-            return device_info.get('path')
-
-        return connection_properties.get('device_path') or ''
-
     @staticmethod
     def requires_flush(path, path_used, was_multipath):
         """Check if a device needs to be flushed when detaching.
@@ -270,23 +272,29 @@ class LinuxSCSI(executor.Executor):
         """
         # No used path happens on failed attachs, when we don't care about
         # individual flushes.
-        # When Nova used a multipath we don't need to do individual flushes.
-        if not path_used or was_multipath:
+        if not path_used:
             return False
 
-        # We need to flush the single path that was used.
-        # For encrypted volumes the symlink has been replaced, so realpath
-        # won't return device under /dev but under /dev/disk/...
         path = os.path.realpath(path)
         path_used = os.path.realpath(path_used)
-        return path_used == path or '/dev' != os.path.split(path_used)[0]
 
-    def remove_connection(self, devices_names, is_multipath, force=False,
-                          exc=None, path_used=None, was_multipath=False):
+        # Need to flush this device if we used this specific path.  We check
+        # this before checking if it's multipath in case we don't detect it
+        # being multipath correctly (as in bug #1897787).
+        if path_used == path:
+            return True
+
+        # We flush individual path if Nova didn't use a multipath and we
+        # replaced the symlink to a real device with a link to the decrypted
+        # DM.  We know we replaced it because it doesn't link to /dev/XYZ,
+        # instead it maps to /dev/mapped/crypt-XYZ
+        return not was_multipath and '/dev' != os.path.split(path_used)[0]
+
+    def remove_connection(self, devices_names, force=False, exc=None,
+                          path_used=None, was_multipath=False):
         """Remove LUNs and multipath associated with devices names.
 
         :param devices_names: Iterable with real device names ('sda', 'sdb')
-        :param is_multipath: Whether this is a multipath connection or not
         :param force: Whether to forcefully disconnect even if flush fails.
         :param exc: ExceptionChainer where to add exceptions if forcing
         :param path_used: What path was used by Nova/Cinder for I/O
@@ -295,22 +303,32 @@ class LinuxSCSI(executor.Executor):
         """
         if not devices_names:
             return
-        multipath_name = None
         exc = exception.ExceptionChainer() if exc is None else exc
-        LOG.debug('Removing %(type)s devices %(devices)s',
-                  {'type': 'multipathed' if is_multipath else 'single pathed',
-                   'devices': ', '.join(devices_names)})
 
-        if is_multipath:
-            multipath_dm = self.find_sysfs_multipath_dm(devices_names)
-            multipath_name = multipath_dm and self.get_dm_name(multipath_dm)
-            if multipath_name:
-                with exc.context(force, 'Flushing %s failed', multipath_name):
-                    self.flush_multipath_device(multipath_name)
-                    multipath_name = None
+        multipath_dm = self.find_sysfs_multipath_dm(devices_names)
+        LOG.debug('Removing %(type)s devices %(devices)s',
+                  {'type': 'multipathed' if multipath_dm else 'single pathed',
+                   'devices': ', '.join(devices_names)})
+        multipath_name = multipath_dm and self.get_dm_name(multipath_dm)
+        if multipath_name:
+            with exc.context(force, 'Flushing %s failed', multipath_name):
+                self.flush_multipath_device(multipath_name)
+                multipath_name = None
+            multipath_running = True
+        else:
+            multipath_running = self.is_multipath_running(
+                enforce_multipath=False, root_helper=self._root_helper)
 
         for device_name in devices_names:
             dev_path = '/dev/' + device_name
+            if multipath_running:
+                # Recent multipathd doesn't remove path devices in time when
+                # it receives mutiple udev events in a short span, so here we
+                # tell multipathd to remove the path device immediately.
+                # Even if this step fails, later removing an iscsi device
+                # triggers a udev event and multipathd can remove the path
+                # device based on the udev event
+                self.multipath_del_path(dev_path)
             flush = self.requires_flush(dev_path, path_used, was_multipath)
             self.remove_scsi_device(dev_path, force, exc, flush)
 
@@ -366,7 +384,7 @@ class LinuxSCSI(executor.Executor):
                       attempts=3, timeout=300, interval=10,
                       root_helper=self._root_helper)
 
-    @utils.retry(exceptions=exception.VolumeDeviceNotFound)
+    @utils.retry(exception.VolumeDeviceNotFound)
     def wait_for_path(self, volume_path):
         """Wait for a path to show up."""
         LOG.debug("Checking to see if %s exists yet.",
@@ -378,7 +396,7 @@ class LinuxSCSI(executor.Executor):
         else:
             LOG.debug("%s has shown up.", volume_path)
 
-    @utils.retry(exceptions=exception.BlockDeviceReadOnly, retries=5)
+    @utils.retry(exception.BlockDeviceReadOnly, retries=5)
     def wait_for_rw(self, wwn, device_path):
         """Wait for block device to be Read-Write."""
         LOG.debug("Checking to see if %s is read-only.",
@@ -480,8 +498,8 @@ class LinuxSCSI(executor.Executor):
                 cmd='multipath -l %s' % device)
 
         if out:
-            lines = out.strip()
-            lines = lines.split("\n")
+            lines_str = out.strip()
+            lines = lines_str.split("\n")
             lines = [line for line in lines
                      if not re.match(MULTIPATH_ERROR_REGEX, line) and
                      len(line)]
@@ -534,17 +552,6 @@ class LinuxSCSI(executor.Executor):
             return info
         return None
 
-    def get_device_size(self, device):
-        """Get the size in bytes of a volume."""
-        (out, _err) = self._execute('blockdev', '--getsize64',
-                                    device, run_as_root=True,
-                                    root_helper=self._root_helper)
-        var = six.text_type(out.strip())
-        if var.isnumeric():
-            return int(var)
-        else:
-            return None
-
     def multipath_reconfigure(self):
         """Issue a multipathd reconfigure.
 
@@ -558,18 +565,48 @@ class LinuxSCSI(executor.Executor):
                                     root_helper=self._root_helper)
         return out
 
+    def _multipath_resize_map(self, mpath_id):
+        cmd = ('multipathd', 'resize', 'map', mpath_id)
+        (out, _err) = self._execute(*cmd,
+                                    run_as_root=True,
+                                    root_helper=self._root_helper)
+        if 'fail' in out or 'timeout' in out:
+            raise putils.ProcessExecutionError(
+                stdout=out, stderr=_err,
+                exit_code=1, cmd=cmd)
+
+        return out
+
     def multipath_resize_map(self, mpath_id):
         """Issue a multipath resize map on device.
 
         This forces the multipath daemon to update it's
         size information a particular multipath device.
         """
-        (out, _err) = self._execute('multipathd', 'resize', 'map', mpath_id,
-                                    run_as_root=True,
-                                    root_helper=self._root_helper)
-        return out
 
-    def extend_volume(self, volume_paths, use_multipath=False):
+        # "multipathd reconfigure" is async since 0.6.1. While the
+        # operation is in progress, "multipathd resize map" returns
+        # "timeout".
+        tstart = time.time()
+        while True:
+            try:
+                self._multipath_resize_map(mpath_id)
+                break
+            except putils.ProcessExecutionError as err:
+                with excutils.save_and_reraise_exception(reraise=True) as ctx:
+                    elapsed = time.time() - tstart
+                    if 'timeout' in err.stdout and (
+                            elapsed < MULTIPATHD_RESIZE_TIMEOUT):
+                        LOG.debug(
+                            "multipathd resize map timed out. "
+                            "Elapsed: %s, timeout: %s. Retrying...",
+                            elapsed, MULTIPATHD_RESIZE_TIMEOUT)
+                        ctx.reraise = False
+                        time.sleep(1)
+
+    def extend_volume(self,
+                      volume_paths: list,
+                      use_multipath: bool = False) -> Optional[int]:
         """Signal the SCSI subsystem to test for volume resize.
 
         This function tries to signal the local system's kernel
@@ -589,13 +626,13 @@ class LinuxSCSI(executor.Executor):
             scsi_path = ("/sys/bus/scsi/drivers/sd/%(device_id)s" %
                          {'device_id': device_id})
 
-            size = self.get_device_size(volume_path)
+            size = utils.get_device_size(self, volume_path)
             LOG.debug("Starting size: %s", size)
 
             # now issue the device rescan
             rescan_path = "%(scsi_path)s/rescan" % {'scsi_path': scsi_path}
             self.echo_scsi_command(rescan_path, "1")
-            new_size = self.get_device_size(volume_path)
+            new_size = utils.get_device_size(self, volume_path)
             LOG.debug("volume size after scsi device rescan %s", new_size)
 
         scsi_wwn = self.get_scsi_wwn(volume_paths[0])
@@ -605,18 +642,13 @@ class LinuxSCSI(executor.Executor):
                 # Force a reconfigure so that resize works
                 self.multipath_reconfigure()
 
-                size = self.get_device_size(mpath_device)
+                size = utils.get_device_size(self, mpath_device)
                 LOG.info("mpath(%(device)s) current size %(size)s",
                          {'device': mpath_device, 'size': size})
-                result = self.multipath_resize_map(scsi_wwn)
-                if 'fail' in result:
-                    LOG.error("Multipathd failed to update the size mapping "
-                              "of multipath device %(scsi_wwn)s volume "
-                              "%(volume)s",
-                              {'scsi_wwn': scsi_wwn, 'volume': volume_paths})
-                    return None
 
-                new_size = self.get_device_size(mpath_device)
+                self.multipath_resize_map(scsi_wwn)
+
+                new_size = utils.get_device_size(self, mpath_device)
                 LOG.info("mpath(%(device)s) new size %(size)s",
                          {'device': mpath_device, 'size': new_size})
 
@@ -719,3 +751,11 @@ class LinuxSCSI(executor.Executor):
                                        check_exit_code=False,
                                        root_helper=self._root_helper)
         return stdout.strip() == 'ok'
+
+    def multipath_del_path(self, realpath):
+        """Remove a path from multipathd for monitoring."""
+        stdout, stderr = self._execute('multipathd', 'del', 'path', realpath,
+                                       run_as_root=True, timeout=5,
+                                       check_exit_code=False,
+                                       root_helper=self._root_helper)
+        return stdout.strip() == 'ok'
