@@ -140,8 +140,8 @@ class WindowsFCConnector(win_conn_base.BaseWindowsConnector):
         for node_name in hba_mappings:
             target_mappings = self._fc_utils.get_fc_target_mappings(node_name)
             for mapping in target_mappings:
-                if (mapping['port_name'] in target_wwpns
-                        and mapping['lun'] == target_lun):
+                if (mapping['port_name'] in target_wwpns and
+                        mapping['lun'] == target_lun):
                     volume_mappings.append(mapping)
 
         return volume_mappings
@@ -162,7 +162,7 @@ class WindowsFCConnector(win_conn_base.BaseWindowsConnector):
                         local_port_wwn, remote_port_wwn, fcp_lun)
 
                     # This may raise a DiskNotFound exception if the disks
-                    # are meanwhile claimed by the MPIO serivce.
+                    # are meanwhile claimed by the MPIO service.
                     disk_paths = [
                         self._diskutils.get_device_name_by_device_number(
                             dev_num)
