@@ -13,24 +13,20 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""
-LVM class for performing LVM operations.
-"""
+"""LVM class for performing LVM operations."""
 
 import math
 import os
 import re
 
-from os_brick import exception
-from os_brick import executor
-from os_brick.i18n import _LE, _LI
-from os_brick.privileged import rootwrap as priv_rootwrap
-from os_brick import utils
 from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
 from oslo_utils import excutils
-from six import moves
 
+from os_brick import exception
+from os_brick import executor
+from os_brick.privileged import rootwrap as priv_rootwrap
+from os_brick import utils
 
 LOG = logging.getLogger(__name__)
 
@@ -42,7 +38,8 @@ class LVM(executor.Executor):
 
     def __init__(self, vg_name, root_helper, create_vg=False,
                  physical_volumes=None, lvm_type='default',
-                 executor=None, lvm_conf=None):
+                 executor=None, lvm_conf=None,
+                 suppress_fd_warn=False):
 
         """Initialize the LVM object.
 
@@ -57,7 +54,7 @@ class LVM(executor.Executor):
         :param lvm_type: VG and Volume type (default, or thin)
         :param executor: Execute method to use, None uses
                          oslo_concurrency.processutils
-
+        :param suppress_fd_warn: Add suppress FD Warn to LVM env
         """
         super(LVM, self).__init__(execute=executor, root_helper=root_helper)
         self.vg_name = vg_name
@@ -76,26 +73,32 @@ class LVM(executor.Executor):
         # Ensure LVM_SYSTEM_DIR has been added to LVM.LVM_CMD_PREFIX
         # before the first LVM command is executed, and use the directory
         # where the specified lvm_conf file is located as the value.
+
+        # NOTE(jdg): We use the temp var here becuase LVM_CMD_PREFIX is a
+        # class global and if you use append here, you'll literally just keep
+        # appending values to the global.
+        _lvm_cmd_prefix = ['env', 'LC_ALL=C']
+
         if lvm_conf and os.path.isfile(lvm_conf):
             lvm_sys_dir = os.path.dirname(lvm_conf)
-            LVM.LVM_CMD_PREFIX = ['env',
-                                  'LC_ALL=C',
-                                  'LVM_SYSTEM_DIR=' + lvm_sys_dir]
+            _lvm_cmd_prefix.append('LVM_SYSTEM_DIR=' + lvm_sys_dir)
+
+        if suppress_fd_warn:
+            _lvm_cmd_prefix.append('LVM_SUPPRESS_FD_WARNINGS=1')
+        LVM.LVM_CMD_PREFIX = _lvm_cmd_prefix
 
         if create_vg and physical_volumes is not None:
-            self.pv_list = physical_volumes
-
             try:
                 self._create_vg(physical_volumes)
             except putils.ProcessExecutionError as err:
-                LOG.exception(_LE('Error creating Volume Group'))
-                LOG.error(_LE('Cmd     :%s'), err.cmd)
-                LOG.error(_LE('StdOut  :%s'), err.stdout)
-                LOG.error(_LE('StdErr  :%s'), err.stderr)
+                LOG.exception('Error creating Volume Group')
+                LOG.error('Cmd     :%s', err.cmd)
+                LOG.error('StdOut  :%s', err.stdout)
+                LOG.error('StdErr  :%s', err.stderr)
                 raise exception.VolumeGroupCreationFailed(vg_name=self.vg_name)
 
         if self._vg_exists() is False:
-            LOG.error(_LE('Unable to locate Volume Group %s'), vg_name)
+            LOG.error('Unable to locate Volume Group %s', vg_name)
             raise exception.VolumeGroupNotFound(vg_name=vg_name)
 
         # NOTE: we assume that the VG has been activated outside of Cinder
@@ -140,6 +143,24 @@ class LVM(executor.Executor):
         cmd = ['vgcreate', self.vg_name, ','.join(pv_list)]
         self._execute(*cmd, root_helper=self._root_helper, run_as_root=True)
 
+    @utils.retry(retry=utils.retry_if_exit_code, retry_param=139, interval=0.5,
+                 backoff_rate=0.5)
+    def _run_lvm_command(self,
+                         cmd_arg_list: list,
+                         root_helper: str = None,
+                         run_as_root: bool = True) -> tuple:
+        """Run LVM commands with a retry on code 139 to work around LVM bugs.
+
+        Refer to LP bug 1901783, LP bug 1932188.
+        """
+        if not root_helper:
+            root_helper = self._root_helper
+
+        (out, err) = self._execute(*cmd_arg_list,
+                                   root_helper=root_helper,
+                                   run_as_root=run_as_root)
+        return (out, err)
+
     def _get_vg_uuid(self):
         cmd = LVM.LVM_CMD_PREFIX + ['vgs', '--noheadings',
                                     '-o', 'uuid', self.vg_name]
@@ -169,9 +190,8 @@ class LVM(executor.Executor):
         free_space = 0.0
 
         try:
-            (out, err) = self._execute(*cmd,
-                                       root_helper=self._root_helper,
-                                       run_as_root=True)
+            (out, err) = self._run_lvm_command(cmd)
+
             if out is not None:
                 out = out.strip()
                 data = out.split(':')
@@ -180,11 +200,14 @@ class LVM(executor.Executor):
                 consumed_space = pool_size / 100 * data_percent
                 free_space = pool_size - consumed_space
                 free_space = round(free_space, 2)
-        except putils.ProcessExecutionError as err:
-            LOG.exception(_LE('Error querying thin pool about data_percent'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+        # Need noqa due to a false error about the 'err' variable being unused
+        # even though it is used in the logging. Possibly related to
+        # https://github.com/PyCQA/pyflakes/issues/378.
+        except putils.ProcessExecutionError as err:  # noqa
+            LOG.exception('Error querying thin pool about data_percent')
+            LOG.error('Cmd     :%s', err.cmd)
+            LOG.error('StdOut  :%s', err.stdout)
+            LOG.error('StdErr  :%s', err.stderr)
 
         return free_space
 
@@ -273,6 +296,8 @@ class LVM(executor.Executor):
             return False
 
     @staticmethod
+    @utils.retry(retry=utils.retry_if_exit_code, retry_param=139, interval=0.5,
+                 backoff_rate=0.5)  # Bug#1901783
     def get_lv_info(root_helper, vg_name=None, lv_name=None):
         """Retrieve info about LVs (all, in a VG, or a single LV).
 
@@ -298,15 +323,15 @@ class LVM(executor.Executor):
             with excutils.save_and_reraise_exception(reraise=True) as ctx:
                 if "not found" in err.stderr or "Failed to find" in err.stderr:
                     ctx.reraise = False
-                    LOG.info(_LI("Logical Volume not found when querying "
-                                 "LVM info. (vg_name=%(vg)s, lv_name=%(lv)s"),
+                    LOG.info("Logical Volume not found when querying "
+                             "LVM info. (vg_name=%(vg)s, lv_name=%(lv)s",
                              {'vg': vg_name, 'lv': lv_name})
                     out = None
 
         lv_list = []
         if out is not None:
             volumes = out.split()
-            iterator = moves.zip(*[iter(volumes)] * 3)  # pylint: disable=E1101
+            iterator = zip(*[iter(volumes)] * 3)  # pylint: disable=E1101
             for vg, name, size in iterator:
                 lv_list.append({"vg": vg, "name": name, "size": size})
 
@@ -421,7 +446,7 @@ class LVM(executor.Executor):
         vg_list = self.get_all_volume_groups(self._root_helper, self.vg_name)
 
         if len(vg_list) != 1:
-            LOG.error(_LE('Unable to find VG: %s'), self.vg_name)
+            LOG.error('Unable to find VG: %s', self.vg_name)
             raise exception.VolumeGroupNotFound(vg_name=self.vg_name)
 
         self.vg_size = float(vg_list[0]['size'])
@@ -464,7 +489,7 @@ class LVM(executor.Executor):
                 if not lv['size'][-1].isdigit():
                     lvsize = lvsize[:-1]
                 if lv['name'] == self.vg_thin_pool:
-                    self.vg_thin_pool_size = lvsize
+                    self.vg_thin_pool_size = float(lvsize)
                     tpfs = self._get_thin_pool_free_space(self.vg_name,
                                                           self.vg_thin_pool)
                     self.vg_thin_pool_free_space = tpfs
@@ -510,9 +535,9 @@ class LVM(executor.Executor):
         """
 
         if not LVM.supports_thin_provisioning(self._root_helper):
-            LOG.error(_LE('Requested to setup thin provisioning, '
-                          'however current LVM version does not '
-                          'support it.'))
+            LOG.error('Requested to setup thin provisioning, '
+                      'however current LVM version does not '
+                      'support it.')
             return None
 
         if name is None:
@@ -531,9 +556,7 @@ class LVM(executor.Executor):
                                       'size': size_args,
                                       'free': self.vg_free_space})
 
-        self._execute(*cmd,
-                      root_helper=self._root_helper,
-                      run_as_root=True)
+        self._run_lvm_command(cmd)
 
         self.vg_thin_pool = name
 
@@ -568,14 +591,12 @@ class LVM(executor.Executor):
                 cmd.extend(['-R', str(rsize)])
 
         try:
-            self._execute(*cmd,
-                          root_helper=self._root_helper,
-                          run_as_root=True)
+            self._run_lvm_command(cmd)
         except putils.ProcessExecutionError as err:
-            LOG.exception(_LE('Error creating Volume'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.exception('Error creating Volume')
+            LOG.error('Cmd     :%s', err.cmd)
+            LOG.error('StdOut  :%s', err.stdout)
+            LOG.error('StdErr  :%s', err.stderr)
             raise
 
     @utils.retry(putils.ProcessExecutionError)
@@ -589,24 +610,23 @@ class LVM(executor.Executor):
         """
         source_lvref = self.get_volume(source_lv_name)
         if source_lvref is None:
-            LOG.error(_LE("Trying to create snapshot by non-existent LV: %s"),
+            LOG.error("Trying to create snapshot by non-existent LV: %s",
                       source_lv_name)
             raise exception.VolumeDeviceNotFound(device=source_lv_name)
-        cmd = LVM.LVM_CMD_PREFIX + ['lvcreate', '--name', name, '--snapshot',
+        cmd = LVM.LVM_CMD_PREFIX + ['lvcreate', '--name', name,
+                                    '-k', 'y', '--snapshot',
                                     '%s/%s' % (self.vg_name, source_lv_name)]
         if lv_type != 'thin':
             size = source_lvref['size']
             cmd.extend(['-L', '%sg' % (size)])
 
         try:
-            self._execute(*cmd,
-                          root_helper=self._root_helper,
-                          run_as_root=True)
+            self._run_lvm_command(cmd)
         except putils.ProcessExecutionError as err:
-            LOG.exception(_LE('Error creating snapshot'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.exception('Error creating snapshot')
+            LOG.error('Cmd     :%s', err.cmd)
+            LOG.error('StdOut  :%s', err.stdout)
+            LOG.error('StdErr  :%s', err.stderr)
             raise
 
     def _mangle_lv_name(self, name):
@@ -619,9 +639,7 @@ class LVM(executor.Executor):
     def _lv_is_active(self, name):
         cmd = LVM.LVM_CMD_PREFIX + ['lvdisplay', '--noheading', '-C', '-o',
                                     'Attr', '%s/%s' % (self.vg_name, name)]
-        out, _err = self._execute(*cmd,
-                                  root_helper=self._root_helper,
-                                  run_as_root=True)
+        out, _err = self._run_lvm_command(cmd)
         if out:
             out = out.strip()
             # An example output might be '-wi-a----'; the 4th index specifies
@@ -639,18 +657,18 @@ class LVM(executor.Executor):
                           root_helper=self._root_helper,
                           run_as_root=True)
         except putils.ProcessExecutionError as err:
-            LOG.exception(_LE('Error deactivating LV'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.exception('Error deactivating LV')
+            LOG.error('Cmd     :%s', err.cmd)
+            LOG.error('StdOut  :%s', err.stdout)
+            LOG.error('StdErr  :%s', err.stderr)
             raise
 
         # Wait until lv is deactivated to return in
         # order to prevent a race condition.
         self._wait_for_volume_deactivation(name)
 
-    @utils.retry(exceptions=exception.VolumeNotDeactivated, retries=3,
-                 backoff_rate=1)
+    @utils.retry(retry_param=exception.VolumeNotDeactivated, retries=5,
+                 backoff_rate=2)
     def _wait_for_volume_deactivation(self, name):
         LOG.debug("Checking to see if volume %s has been deactivated.",
                   name)
@@ -696,10 +714,10 @@ class LVM(executor.Executor):
                           root_helper=self._root_helper,
                           run_as_root=True)
         except putils.ProcessExecutionError as err:
-            LOG.exception(_LE('Error activating LV'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.exception('Error activating LV')
+            LOG.error('Cmd     :%s', err.cmd)
+            LOG.error('StdOut  :%s', err.stdout)
+            LOG.error('StdErr  :%s', err.stderr)
             raise
 
     @utils.retry(putils.ProcessExecutionError)
@@ -761,10 +779,9 @@ class LVM(executor.Executor):
 
     def lv_has_snapshot(self, name):
         cmd = LVM.LVM_CMD_PREFIX + ['lvdisplay', '--noheading', '-C', '-o',
-                                    'Attr', '%s/%s' % (self.vg_name, name)]
-        out, _err = self._execute(*cmd,
-                                  root_helper=self._root_helper,
-                                  run_as_root=True)
+                                    'Attr', '--readonly',
+                                    '%s/%s' % (self.vg_name, name)]
+        out, _err = self._run_lvm_command(cmd)
         if out:
             out = out.strip()
             if (out[0] == 'o') or (out[0] == 'O'):
@@ -785,10 +802,10 @@ class LVM(executor.Executor):
             self._execute(*cmd, root_helper=self._root_helper,
                           run_as_root=True)
         except putils.ProcessExecutionError as err:
-            LOG.exception(_LE('Error extending Volume'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.exception('Error extending Volume')
+            LOG.error('Cmd     :%s', err.cmd)
+            LOG.error('StdOut  :%s', err.stdout)
+            LOG.error('StdErr  :%s', err.stderr)
             raise
 
     def vg_mirror_free_space(self, mirror_count):
@@ -823,8 +840,8 @@ class LVM(executor.Executor):
                           root_helper=self._root_helper,
                           run_as_root=True)
         except putils.ProcessExecutionError as err:
-            LOG.exception(_LE('Error renaming logical volume'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.exception('Error renaming logical volume')
+            LOG.error('Cmd     :%s', err.cmd)
+            LOG.error('StdOut  :%s', err.stdout)
+            LOG.error('StdErr  :%s', err.stderr)
             raise
