@@ -119,13 +119,27 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
         res = self.connector._get_iscsi_nodes()
         self.assertEqual([], res)
 
+    @mock.patch.object(iscsi.ISCSIConnector, '_execute')
+    def test_get_iscsi_nodes_corrupt(self, exec_mock):
+        iscsiadm_result = ('ip1:port1,-1 tgt1\n'
+                           'ip2:port2,-1 tgt2\n'
+                           '[]:port3,-1\n'
+                           'ip4:port4,-1 tgt4\n')
+        exec_mock.return_value = (iscsiadm_result, '')
+        res = self.connector._get_iscsi_nodes()
+        expected = [('ip1:port1', 'tgt1'), ('ip2:port2', 'tgt2'),
+                    ('ip4:port4', 'tgt4')]
+        self.assertListEqual(expected, res)
+        exec_mock.assert_called_once_with(
+            'iscsiadm', '-m', 'node', run_as_root=True,
+            root_helper=self.connector._root_helper, check_exit_code=False)
+
     @mock.patch.object(iscsi.ISCSIConnector, '_get_ips_iqns_luns')
     @mock.patch('glob.glob')
     @mock.patch.object(iscsi.ISCSIConnector, '_get_iscsi_sessions_full')
     @mock.patch.object(iscsi.ISCSIConnector, '_get_iscsi_nodes')
     def test_get_connection_devices(self, nodes_mock, sessions_mock,
                                     glob_mock, iql_mock):
-        self.connector.use_multipath = True
         iql_mock.return_value = self.connector._get_all_targets(self.CON_PROPS)
 
         # List sessions from other targets and non tcp sessions
@@ -151,7 +165,8 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
                     ('ip2:port2', 'tgt2'): ({'sdb'}, {'sdc'}),
                     ('ip3:port3', 'tgt3'): (set(), set())}
         self.assertDictEqual(expected, res)
-        iql_mock.assert_called_once_with(self.CON_PROPS, discover=False)
+        iql_mock.assert_called_once_with(self.CON_PROPS, discover=False,
+                                         is_disconnect_call=False)
 
     @mock.patch('glob.glob')
     @mock.patch.object(iscsi.ISCSIConnector, '_get_iscsi_sessions_full')
@@ -536,16 +551,21 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
 
     @mock.patch.object(iscsi.ISCSIConnector, '_cleanup_connection')
     def test_disconnect_volume(self, cleanup_mock):
-        res = self.connector.disconnect_volume(mock.sentinel.con_props,
+        conn_props = {'target_portal': '198.72.124.185:3260',
+                      'target_iqn': 'iqn.2010-10.org.openstack:volume-uuid',
+                      'target_lun': 0,
+                      'device_path': '/dev/sda'}
+        res = self.connector.disconnect_volume(conn_props,
                                                mock.sentinel.dev_info,
                                                mock.sentinel.Force,
                                                mock.sentinel.ignore_errors)
         self.assertEqual(cleanup_mock.return_value, res)
         cleanup_mock.assert_called_once_with(
-            mock.sentinel.con_props,
+            conn_props,
             force=mock.sentinel.Force,
             ignore_errors=mock.sentinel.ignore_errors,
-            device_info=mock.sentinel.dev_info)
+            device_info=mock.sentinel.dev_info,
+            is_disconnect_call=True)
 
     @ddt.data(True, False)
     @mock.patch.object(iscsi.ISCSIConnector, '_get_transport')
@@ -661,7 +681,7 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
               ('/dev/dm-11', True),
               ('/dev/disk/by-id/dm-uuid-mpath-MPATH', True))
     @ddt.unpack
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dev_path')
+    @mock.patch('os_brick.utils.get_dev_path')
     @mock.patch.object(iscsi.ISCSIConnector, '_disconnect_connection')
     @mock.patch.object(iscsi.ISCSIConnector, '_get_connection_devices')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'flush_multipath_device')
@@ -677,19 +697,17 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
             (('ip2:port2', 'tgt2'), ({'sdb'}, {'sdc'})),
             (('ip3:port3', 'tgt3'), (set(), set()))))
 
-        with mock.patch.object(self.connector,
-                               'use_multipath') as use_mp_mock:
-            self.connector._cleanup_connection(
-                self.CON_PROPS, ips_iqns_luns=mock.sentinel.ips_iqns_luns,
-                force=False, ignore_errors=False,
-                device_info=mock.sentinel.device_info)
+        self.connector._cleanup_connection(
+            self.CON_PROPS, ips_iqns_luns=mock.sentinel.ips_iqns_luns,
+            force=False, ignore_errors=False,
+            device_info=mock.sentinel.device_info)
 
         get_dev_path_mock.called_once_with(self.CON_PROPS,
                                            mock.sentinel.device_info)
         con_devs_mock.assert_called_once_with(self.CON_PROPS,
-                                              mock.sentinel.ips_iqns_luns)
-        remove_mock.assert_called_once_with({'sda', 'sdb'}, use_mp_mock,
-                                            False, mock.ANY,
+                                              mock.sentinel.ips_iqns_luns,
+                                              False)
+        remove_mock.assert_called_once_with({'sda', 'sdb'}, False, mock.ANY,
                                             path_used, was_multipath)
         discon_mock.assert_called_once_with(
             self.CON_PROPS,
@@ -715,17 +733,16 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
             (('ip2:port2', 'tgt2'), ({'sdb'}, {'sdc'})),
             (('ip3:port3', 'tgt3'), (set(), set()))))
 
-        with mock.patch.object(self.connector, 'use_multipath',
-                               wraps=True) as use_mp_mock:
-            self.assertRaises(exception.ExceptionChainer,
-                              self.connector._cleanup_connection,
-                              self.CON_PROPS,
-                              ips_iqns_luns=mock.sentinel.ips_iqns_luns,
-                              force=mock.sentinel.force, ignore_errors=False)
+        self.assertRaises(exception.ExceptionChainer,
+                          self.connector._cleanup_connection,
+                          self.CON_PROPS,
+                          ips_iqns_luns=mock.sentinel.ips_iqns_luns,
+                          force=mock.sentinel.force, ignore_errors=False)
 
         con_devs_mock.assert_called_once_with(self.CON_PROPS,
-                                              mock.sentinel.ips_iqns_luns)
-        remove_mock.assert_called_once_with({'sda', 'sdb'}, use_mp_mock,
+                                              mock.sentinel.ips_iqns_luns,
+                                              False)
+        remove_mock.assert_called_once_with({'sda', 'sdb'},
                                             mock.sentinel.force, mock.ANY,
                                             '', False)
         discon_mock.assert_called_once_with(
@@ -961,6 +978,21 @@ Setting up iSCSI targets: unused
         db_portals_mock.assert_called_once_with(self.SINGLE_CON_PROPS)
         discover_mock.assert_not_called()
 
+    @mock.patch.object(iscsi.ISCSIConnector, '_get_all_targets')
+    @mock.patch.object(iscsi.ISCSIConnector, '_get_discoverydb_portals')
+    @mock.patch.object(iscsi.ISCSIConnector, '_discover_iscsi_portals')
+    def test_get_ips_iqns_luns_disconnect_single_path(self, discover_mock,
+                                                      db_portals_mock,
+                                                      get_targets_mock):
+        db_portals_mock.side_effect = exception.TargetPortalsNotFound
+        res = self.connector._get_ips_iqns_luns(self.SINGLE_CON_PROPS,
+                                                discover=False,
+                                                is_disconnect_call=True)
+        db_portals_mock.assert_called_once_with(self.SINGLE_CON_PROPS)
+        discover_mock.assert_not_called()
+        get_targets_mock.assert_called_once_with(self.SINGLE_CON_PROPS)
+        self.assertEqual(get_targets_mock.return_value, res)
+
     @mock.patch.object(iscsi.ISCSIConnector, '_discover_iscsi_portals')
     def test_get_ips_iqns_luns_no_target_iqns_share_iqn(self, discover_mock):
         discover_mock.return_value = [('ip1:port1', 'tgt1', '1'),
@@ -1085,6 +1117,38 @@ Setting up iSCSI targets: unused
         self.assertListEqual(expected_cmds, actual_cmds)
         get_sessions_mock.assert_called_once_with()
 
+    @mock.patch.object(iscsi.ISCSIConnector, '_iscsiadm_update')
+    @mock.patch.object(iscsi.ISCSIConnector, '_get_transport',
+                       return_value='default')
+    @mock.patch.object(iscsi.ISCSIConnector, '_get_iscsi_sessions_full')
+    @mock.patch('os_brick.utils._time_sleep')
+    def test_connect_to_iscsi_portal_fail_op_new(self, sleep_mock,
+                                                 get_sessions_mock,
+                                                 get_transport_mock,
+                                                 iscsiadm_update_mock):
+        get_sessions_mock.return_value = []
+        with mock.patch.object(self.connector, '_execute') as exec_mock:
+            exec_mock.side_effect = [('', 21), ('', 6), ('', 21), ('', 6),
+                                     ('', 21), ('', 6)]
+            self.assertRaises(exception.BrickException,
+                              self.connector._connect_to_iscsi_portal,
+                              self.CON_PROPS)
+        expected_cmds = ['iscsiadm -m node -T tgt1 -p ip1:port1',
+                         'iscsiadm -m node -T tgt1 -p ip1:port1 '
+                         '--interface default --op new',
+                         'iscsiadm -m node -T tgt1 -p ip1:port1',
+                         'iscsiadm -m node -T tgt1 -p ip1:port1 '
+                         '--interface default --op new',
+                         'iscsiadm -m node -T tgt1 -p ip1:port1',
+                         'iscsiadm -m node -T tgt1 -p ip1:port1 '
+                         '--interface default --op new']
+        actual_cmds = [' '.join(args[0]) for args in exec_mock.call_args_list]
+        self.assertListEqual(expected_cmds, actual_cmds)
+        iscsiadm_update_mock.assert_not_called()
+
+        # Called twice by the retry mechanism
+        self.assertEqual(2, sleep_mock.call_count)
+
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwn',
                        side_effect=(None, 'tgt2'))
     @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
@@ -1108,7 +1172,7 @@ Setting up iSCSI targets: unused
         cleanup_mock.assert_called_once_with(
             {'target_lun': 4, 'volume_id': 'vol_id',
              'target_portal': 'ip1:port1', 'target_iqn': 'tgt1'},
-            (('ip1:port1', 'tgt1', 4),),
+            [('ip1:port1', 'tgt1', 4), ],
             force=True, ignore_errors=True)
 
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwn', return_value='')
@@ -1159,7 +1223,7 @@ Setting up iSCSI targets: unused
         calls_per_try = [
             mock.call({'target_portal': prop[0], 'target_iqn': prop[1],
                        'target_lun': prop[2], 'volume_id': 'vol_id'},
-                      (prop,), force=True, ignore_errors=True)
+                      [prop, ], force=True, ignore_errors=True)
             for prop in props
         ]
         cleanup_mock.assert_has_calls(calls_per_try * 3)
@@ -1234,7 +1298,9 @@ Setting up iSCSI targets: unused
 
         connect_mock.side_effect = my_connect
 
-        res = self.connector._connect_multipath_volume(self.CON_PROPS)
+        with mock.patch.object(self.connector,
+                               'use_multipath'):
+            res = self.connector._connect_multipath_volume(self.CON_PROPS)
 
         expected = {'type': 'block', 'scsi_wwn': '', 'multipath_id': '',
                     'path': '/dev/dm-0'}
@@ -1503,6 +1569,18 @@ Setting up iSCSI targets: unused
         expected.update(failed_logins=1, stopped_threads=1)
         self.assertDictEqual(expected, data)
 
+    @mock.patch.object(iscsi.ISCSIConnector, '_connect_to_iscsi_portal')
+    def test_connect_vol_with_connection_failure(self, connect_mock):
+        data = self._get_connect_vol_data()
+
+        connect_mock.side_effect = Exception()
+
+        self.connector._connect_vol(3, self.CON_PROPS, data)
+
+        expected = self._get_connect_vol_data()
+        expected.update(failed_logins=1, stopped_threads=1)
+        self.assertDictEqual(expected, data)
+
     @mock.patch('os_brick.utils._time_sleep', mock.Mock())
     @mock.patch.object(linuxscsi.LinuxSCSI, 'scan_iscsi')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'device_name_by_hctl',
@@ -1567,17 +1645,14 @@ Setting up iSCSI targets: unused
         scan_mock.assert_not_called()
         dev_name_mock.assert_called_once_with(mock.sentinel.session, hctl)
 
-    @mock.patch.object(iscsi.ISCSIConnector, '_get_device_link')
-    def test__get_connect_result(self, get_link_mock):
+    def test__get_connect_result(self):
         props = self.CON_PROPS.copy()
         props['encrypted'] = False
         res = self.connector._get_connect_result(props, 'wwn', ['sda', 'sdb'])
         expected = {'type': 'block', 'scsi_wwn': 'wwn', 'path': '/dev/sda'}
         self.assertDictEqual(expected, res)
-        get_link_mock.assert_not_called()
 
-    @mock.patch.object(iscsi.ISCSIConnector, '_get_device_link')
-    def test__get_connect_result_mpath(self, get_link_mock):
+    def test__get_connect_result_mpath(self):
         props = self.CON_PROPS.copy()
         props['encrypted'] = False
         res = self.connector._get_connect_result(props, 'wwn', ['sda', 'sdb'],
@@ -1585,106 +1660,6 @@ Setting up iSCSI targets: unused
         expected = {'type': 'block', 'scsi_wwn': 'wwn', 'path': '/dev/mpath',
                     'multipath_id': 'wwn'}
         self.assertDictEqual(expected, res)
-        get_link_mock.assert_not_called()
-
-    @mock.patch.object(iscsi.ISCSIConnector, '_get_device_link',
-                       return_value='/dev/disk/by-id/scsi-wwn')
-    def test__get_connect_result_encrypted(self, get_link_mock):
-        props = self.CON_PROPS.copy()
-        props['encrypted'] = True
-        res = self.connector._get_connect_result(props, 'wwn', ['sda', 'sdb'])
-        expected = {'type': 'block', 'scsi_wwn': 'wwn',
-                    'path': get_link_mock.return_value}
-        self.assertDictEqual(expected, res)
-        get_link_mock.assert_called_once_with('wwn', '/dev/sda', None)
-
-    @mock.patch('os.path.realpath', return_value='/dev/sda')
-    def test__get_device_link(self, realpath_mock):
-        symlink = '/dev/disk/by-id/scsi-wwn'
-        res = self.connector._get_device_link('wwn', '/dev/sda', None)
-        self.assertEqual(symlink, res)
-        realpath_mock.assert_called_once_with(symlink)
-
-    @mock.patch('os.path.realpath', return_value='/dev/dm-0')
-    def test__get_device_link_multipath(self, realpath_mock):
-        symlink = '/dev/disk/by-id/dm-uuid-mpath-wwn'
-        res = self.connector._get_device_link('wwn', '/dev/dm-0', 'wwn')
-        self.assertEqual(symlink, res)
-        realpath_mock.assert_called_once_with(symlink)
-
-    @mock.patch('os.path.realpath', side_effect=('/dev/sdz', '/dev/sdy',
-                                                 '/dev/sda', '/dev/sdx'))
-    @mock.patch('os.listdir', return_value=['dm-...', 'scsi-wwn', 'scsi-...'])
-    def test__get_device_link_check_links(self, listdir_mock, realpath_mock):
-        res = self.connector._get_device_link('wwn', '/dev/sda', None)
-        self.assertEqual(res, '/dev/disk/by-id/scsi-wwn')
-        listdir_mock.assert_called_once_with('/dev/disk/by-id/')
-        realpath_mock.assert_has_calls([
-            mock.call('/dev/disk/by-id/scsi-wwn'),
-            mock.call('/dev/disk/by-id/dm-...'),
-            mock.call('/dev/disk/by-id/scsi-wwn')])
-
-    @mock.patch('os_brick.utils._time_sleep')
-    @mock.patch('os.path.realpath', return_value='/dev/sdz')
-    @mock.patch('os.listdir', return_value=['dm-...', 'scsi-...'])
-    def test__get_device_link_not_found(self, listdir_mock, realpath_mock,
-                                        mock_time):
-        self.assertRaises(exception.VolumeDeviceNotFound,
-                          self.connector._get_device_link,
-                          'wwn', '/dev/sda', None)
-        listdir_mock.assert_has_calls(3 * [mock.call('/dev/disk/by-id/')])
-        self.assertEqual(3, listdir_mock.call_count)
-        realpath_mock.assert_has_calls(
-            3 * [mock.call('/dev/disk/by-id/scsi-wwn'),
-                 mock.call('/dev/disk/by-id/dm-...'),
-                 mock.call('/dev/disk/by-id/scsi-...')])
-        self.assertEqual(9, realpath_mock.call_count)
-
-    @mock.patch('os_brick.utils._time_sleep')
-    @mock.patch('os.path.realpath')
-    @mock.patch('os.listdir', return_value=['dm-...', 'scsi-...'])
-    def test__get_device_link_symlink_found_after_retry(self, mock_listdir,
-                                                        mock_realpath,
-                                                        mock_time):
-        # Return the expected realpath on the third retry
-        mock_realpath.side_effect = [
-            None, None, None, None, None, None, '/dev/sda']
-
-        # Assert that VolumeDeviceNotFound isn't raised
-        self.connector._get_device_link('wwn', '/dev/sda', None)
-
-        # Assert that listdir and realpath have been called correctly
-        mock_listdir.assert_has_calls(2 * [mock.call('/dev/disk/by-id/')])
-        self.assertEqual(2, mock_listdir.call_count)
-        mock_realpath.assert_has_calls(
-            2 * [mock.call('/dev/disk/by-id/scsi-wwn'),
-                 mock.call('/dev/disk/by-id/dm-...'),
-                 mock.call('/dev/disk/by-id/scsi-...')]
-            + [mock.call('/dev/disk/by-id/scsi-wwn')])
-        self.assertEqual(7, mock_realpath.call_count)
-
-    @mock.patch('os_brick.utils._time_sleep')
-    @mock.patch('os.path.realpath')
-    @mock.patch('os.listdir', return_value=['dm-...', 'scsi-...'])
-    def test__get_device_link_symlink_found_after_retry_by_listdir(
-            self, mock_listdir, mock_realpath, mock_time):
-
-        # Return the expected realpath on the second retry while looping over
-        # the devices returned by listdir
-        mock_realpath.side_effect = [
-            None, None, None, None, None, '/dev/sda']
-
-        # Assert that VolumeDeviceNotFound isn't raised
-        self.connector._get_device_link('wwn', '/dev/sda', None)
-
-        # Assert that listdir and realpath have been called correctly
-        mock_listdir.assert_has_calls(2 * [mock.call('/dev/disk/by-id/')])
-        self.assertEqual(2, mock_listdir.call_count)
-        mock_realpath.assert_has_calls(
-            2 * [mock.call('/dev/disk/by-id/scsi-wwn'),
-                 mock.call('/dev/disk/by-id/dm-...'),
-                 mock.call('/dev/disk/by-id/scsi-...')])
-        self.assertEqual(6, mock_realpath.call_count)
 
     @mock.patch.object(iscsi.ISCSIConnector, '_run_iscsiadm_bare')
     def test_get_node_startup_values(self, run_iscsiadm_bare_mock):
