@@ -13,10 +13,9 @@
 #    under the License.
 import json
 import os
-import requests
 from unittest import mock
 
-import six
+import requests
 
 from os_brick import exception
 from os_brick.initiator.connectors import scaleio
@@ -110,7 +109,7 @@ class ScaleIOConnectorTestCase(test_connector.ConnectorTestCase):
             self.status_code = status_code
 
         def json(self, **kwargs):
-            if isinstance(self._content, six.string_types):
+            if isinstance(self._content, str):
                 return super(ScaleIOConnectorTestCase.MockHTTPSResponse,
                              self).json(**kwargs)
 
@@ -118,7 +117,7 @@ class ScaleIOConnectorTestCase(test_connector.ConnectorTestCase):
 
         @property
         def text(self):
-            if not isinstance(self._content, six.string_types):
+            if not isinstance(self._content, str):
                 return json.dumps(self._content)
 
             self._content = self._content.encode('utf-8')
@@ -307,7 +306,7 @@ class ScaleIOConnectorTestCase(test_connector.ConnectorTestCase):
 
     @mock.patch.object(os.path, 'exists', return_value=True)
     @mock.patch.object(scaleio.ScaleIOConnector, '_find_volume_path')
-    @mock.patch.object(scaleio.ScaleIOConnector, 'get_device_size')
+    @mock.patch('os_brick.utils.get_device_size')
     def test_extend_volume(self,
                            mock_device_size,
                            mock_find_volume_path,
@@ -320,3 +319,14 @@ class ScaleIOConnectorTestCase(test_connector.ConnectorTestCase):
                          mock_device_size.return_value)
         self.rescan_vols_mock.assert_called_once_with(
             self.connector.RESCAN_VOLS_OP_CODE)
+
+    def test_connection_properties_without_failed_over(self):
+        """Handle connection properties with 'failed_over' missing"""
+        connection_properties = dict(self.fake_connection_properties)
+        connection_properties.pop('failed_over')
+
+        self.connector.connect_volume(connection_properties)
+        self.get_password_mock.assert_called_once_with(
+            scaleio.CONNECTOR_CONF_PATH,
+            connection_properties['config_group'],
+            False)
