@@ -142,7 +142,7 @@ class LinuxSCSITestCase(base.TestCase):
         wwn = self.linuxscsi.get_scsi_wwn(fake_path)
         self.assertEqual(fake_wwn, wwn)
 
-    @mock.patch('six.moves.builtins.open')
+    @mock.patch('builtins.open')
     def test_get_dm_name(self, open_mock):
         dm_map_name = '3600d0230000000000e13955cc3757800'
         cm_open = open_mock.return_value.__enter__.return_value
@@ -151,7 +151,7 @@ class LinuxSCSITestCase(base.TestCase):
         self.assertEqual(dm_map_name, res)
         open_mock.assert_called_once_with('/sys/block/dm-0/dm/name')
 
-    @mock.patch('six.moves.builtins.open', side_effect=IOError)
+    @mock.patch('builtins.open', side_effect=IOError)
     def test_get_dm_name_failure(self, open_mock):
         self.assertEqual('', self.linuxscsi.get_dm_name('dm-0'))
 
@@ -229,15 +229,20 @@ class LinuxSCSITestCase(base.TestCase):
               {'do_raise': True, 'force': True})
     @ddt.unpack
     @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_path')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'is_multipath_running',
+                       return_value=True)
     @mock.patch.object(linuxscsi.LinuxSCSI, 'flush_multipath_device')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_volumes_removal')
-    @mock.patch('os_brick.initiator.linuxscsi.LinuxSCSI.remove_scsi_device')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
     def test_remove_connection_multipath_complete(self, remove_mock, wait_mock,
                                                   find_dm_mock,
                                                   get_dm_name_mock,
                                                   flush_mp_mock,
+                                                  is_mp_running_mock,
+                                                  mp_del_path_mock,
                                                   remove_link_mock,
                                                   do_raise, force):
         if do_raise:
@@ -245,7 +250,6 @@ class LinuxSCSITestCase(base.TestCase):
         devices_names = ('sda', 'sdb')
         exc = exception.ExceptionChainer()
         mp_name = self.linuxscsi.remove_connection(devices_names,
-                                                   is_multipath=True,
                                                    force=mock.sentinel.Force,
                                                    exc=exc)
         find_dm_mock.assert_called_once_with(devices_names)
@@ -253,6 +257,9 @@ class LinuxSCSITestCase(base.TestCase):
         flush_mp_mock.assert_called_once_with(get_dm_name_mock.return_value)
         self.assertEqual(get_dm_name_mock.return_value if do_raise else None,
                          mp_name)
+        is_mp_running_mock.assert_not_called()
+        mp_del_path_mock.assert_has_calls([
+            mock.call('/dev/sda'), mock.call('/dev/sdb')])
         remove_mock.assert_has_calls([
             mock.call('/dev/sda', mock.sentinel.Force, exc, False),
             mock.call('/dev/sdb', mock.sentinel.Force, exc, False)])
@@ -261,6 +268,46 @@ class LinuxSCSITestCase(base.TestCase):
         remove_link_mock.assert_called_once_with(devices_names)
 
     @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_path')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'is_multipath_running',
+                       return_value=True)
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'flush_multipath_device')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm',
+                       return_value=None)
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_volumes_removal')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
+    def test_remove_connection_multipath_complete_no_dm(self, remove_mock,
+                                                        wait_mock,
+                                                        find_dm_mock,
+                                                        get_dm_name_mock,
+                                                        flush_mp_mock,
+                                                        is_mp_running_mock,
+                                                        mp_del_path_mock,
+                                                        remove_link_mock):
+        devices_names = ('sda', 'sdb')
+        exc = exception.ExceptionChainer()
+        mp_name = self.linuxscsi.remove_connection(devices_names,
+                                                   force=mock.sentinel.Force,
+                                                   exc=exc)
+        find_dm_mock.assert_called_once_with(devices_names)
+        get_dm_name_mock.assert_not_called()
+        flush_mp_mock.assert_not_called()
+        self.assertIsNone(mp_name)
+        is_mp_running_mock.assert_called_once()
+        mp_del_path_mock.assert_has_calls([
+            mock.call('/dev/sda'), mock.call('/dev/sdb')])
+        remove_mock.assert_has_calls([
+            mock.call('/dev/sda', mock.sentinel.Force, exc, False),
+            mock.call('/dev/sdb', mock.sentinel.Force, exc, False)])
+        wait_mock.assert_called_once_with(devices_names)
+        self.assertFalse(bool(exc))
+        remove_link_mock.assert_called_once_with(devices_names)
+
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_path')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'is_multipath_running',
+                       return_value=True)
     @mock.patch.object(linuxscsi.LinuxSCSI, 'flush_multipath_device',
                        side_effect=Exception)
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name')
@@ -269,32 +316,51 @@ class LinuxSCSITestCase(base.TestCase):
     @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
     def test_remove_connection_multipath_fail(self, remove_mock, wait_mock,
                                               find_dm_mock, get_dm_name_mock,
-                                              flush_mp_mock, remove_link_mock):
+                                              flush_mp_mock,
+                                              is_mp_running_mock,
+                                              mp_del_path_mock,
+                                              remove_link_mock):
         flush_mp_mock.side_effect = exception.ExceptionChainer
         devices_names = ('sda', 'sdb')
         exc = exception.ExceptionChainer()
         self.assertRaises(exception.ExceptionChainer,
                           self.linuxscsi.remove_connection,
-                          devices_names, is_multipath=True,
-                          force=False, exc=exc)
+                          devices_names, force=False, exc=exc)
         find_dm_mock.assert_called_once_with(devices_names)
         get_dm_name_mock.assert_called_once_with(find_dm_mock.return_value)
         flush_mp_mock.assert_called_once_with(get_dm_name_mock.return_value)
+        is_mp_running_mock.assert_not_called()
+        mp_del_path_mock.assert_not_called()
         remove_mock.assert_not_called()
         wait_mock.assert_not_called()
         remove_link_mock.assert_not_called()
         self.assertTrue(bool(exc))
 
     @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_path')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'is_multipath_running',
+                       return_value=True)
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_volumes_removal')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
-    def test_remove_connection_singlepath(self, remove_mock, wait_mock,
-                                          remove_link_mock):
+    def test_remove_connection_singlepath_no_path(self, remove_mock, wait_mock,
+                                                  find_dm_mock,
+                                                  is_mp_running_mock,
+                                                  mp_del_path_mock,
+                                                  remove_link_mock):
+        # Test remove connection when we didn't form a multipath and didn't
+        # even use any of the devices that were found.  This means that we
+        # don't flush any of the single paths when removing them.
+        find_dm_mock.return_value = None
         devices_names = ('sda', 'sdb')
         exc = exception.ExceptionChainer()
-        self.linuxscsi.remove_connection(devices_names, is_multipath=False,
+        self.linuxscsi.remove_connection(devices_names,
                                          force=mock.sentinel.Force,
                                          exc=exc)
+        find_dm_mock.assert_called_once_with(devices_names)
+        is_mp_running_mock.assert_called_once()
+        mp_del_path_mock.assert_has_calls([
+            mock.call('/dev/sda'), mock.call('/dev/sdb')])
         remove_mock.assert_has_calls(
             [mock.call('/dev/sda', mock.sentinel.Force, exc, False),
              mock.call('/dev/sdb', mock.sentinel.Force, exc, False)])
@@ -302,19 +368,33 @@ class LinuxSCSITestCase(base.TestCase):
         remove_link_mock.assert_called_once_with(devices_names)
 
     @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_path')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'is_multipath_running',
+                       return_value=False)
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_volumes_removal')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
     def test_remove_connection_singlepath_used(self, remove_mock, wait_mock,
+                                               find_dm_mock,
+                                               is_mp_running_mock,
+                                               mp_del_path_mock,
                                                remove_link_mock):
+        # Test remove connection when we didn't form a multipath and just used
+        # one of the single paths that were found.  This means that we don't
+        # flush any of the single paths when removing them.
+        find_dm_mock.return_value = None
         devices_names = ('sda', 'sdb')
         exc = exception.ExceptionChainer()
 
         # realpath was mocked on test setup
         with mock.patch('os.path.realpath', side_effect=self.realpath):
-            self.linuxscsi.remove_connection(devices_names, is_multipath=True,
+            self.linuxscsi.remove_connection(devices_names,
                                              force=mock.sentinel.Force,
                                              exc=exc, path_used='/dev/sdb',
                                              was_multipath=False)
+        find_dm_mock.assert_called_once_with(devices_names)
+        is_mp_running_mock.assert_called_once()
+        mp_del_path_mock.assert_not_called()
         remove_mock.assert_has_calls(
             [mock.call('/dev/sda', mock.sentinel.Force, exc, False),
              mock.call('/dev/sdb', mock.sentinel.Force, exc, True)])
@@ -679,26 +759,6 @@ loop0                                     0"""
         self.assertEqual("0", info['devices'][1]['id'])
         self.assertEqual("3", info['devices'][1]['lun'])
 
-    def test_get_device_size(self):
-        mock_execute = mock.Mock()
-        self.linuxscsi._execute = mock_execute
-        size = '1024'
-        mock_execute.return_value = (size, None)
-
-        ret_size = self.linuxscsi.get_device_size('/dev/fake')
-        self.assertEqual(int(size), ret_size)
-
-        size = 'junk'
-        mock_execute.return_value = (size, None)
-        ret_size = self.linuxscsi.get_device_size('/dev/fake')
-        self.assertIsNone(ret_size)
-
-        size_bad = '1024\n'
-        size_good = 1024
-        mock_execute.return_value = (size_bad, None)
-        ret_size = self.linuxscsi.get_device_size('/dev/fake')
-        self.assertEqual(size_good, ret_size)
-
     def test_multipath_reconfigure(self):
         self.linuxscsi.multipath_reconfigure()
         expected_commands = ['multipathd reconfigure']
@@ -712,7 +772,7 @@ loop0                                     0"""
 
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_size')
+    @mock.patch('os_brick.utils.get_device_size')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
     def test_extend_volume_no_mpath(self, mock_device_info,
                                     mock_device_size,
@@ -742,7 +802,7 @@ loop0                                     0"""
 
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_size')
+    @mock.patch('os_brick.utils.get_device_size')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
     def test_extend_volume_with_mpath(self, mock_device_info,
                                       mock_device_size,
@@ -771,10 +831,10 @@ loop0                                     0"""
                          'multipathd resize map %s' % wwn]
         self.assertEqual(expected_cmds, self.cmds)
 
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_resize_map')
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_multipath_resize_map')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_size')
+    @mock.patch('os_brick.utils.get_device_size')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
     def test_extend_volume_with_mpath_fail(self, mock_device_info,
                                            mock_device_size,
@@ -793,11 +853,14 @@ loop0                                     0"""
         mock_find_mpath_path.return_value = ('/dev/mapper/dm-uuid-mpath-%s' %
                                              wwn)
 
-        mock_mpath_resize_map.return_value = 'fail'
+        mock_mpath_resize_map.side_effect = putils.ProcessExecutionError(
+            stdout="fail")
 
-        ret_size = self.linuxscsi.extend_volume(['/dev/fake1', '/dev/fake2'],
-                                                use_multipath=True)
-        self.assertIsNone(ret_size)
+        self.assertRaises(
+            putils.ProcessExecutionError,
+            self.linuxscsi.extend_volume,
+            volume_paths=['/dev/fake1', '/dev/fake2'],
+            use_multipath=True)
 
         # because we don't mock out the echo_scsi_command
         expected_cmds = ['tee -a /sys/bus/scsi/drivers/sd/0:0:0:1/rescan',
@@ -805,6 +868,91 @@ loop0                                     0"""
                          'multipathd reconfigure']
         self.assertEqual(expected_cmds, self.cmds)
 
+    @mock.patch('time.sleep')
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_multipath_resize_map')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device_path')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
+    @mock.patch('os_brick.utils.get_device_size')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
+    def test_extend_volume_with_mpath_pending(self, mock_device_info,
+                                              mock_device_size,
+                                              mock_scsi_wwn,
+                                              mock_find_mpath_path,
+                                              mock_mpath_resize_map,
+                                              mock_sleep):
+        """Test extending a volume where there is a multipath device fail."""
+        mock_device_info.side_effect = [{'host': host,
+                                         'channel': '0',
+                                         'id': '0',
+                                         'lun': '1'} for host in ['0', '1']]
+
+        mock_device_size.side_effect = [1024, 2048, 1024, 2048, 1024, 2048]
+        wwn = '1234567890123456'
+        mock_scsi_wwn.return_value = wwn
+        mock_find_mpath_path.return_value = ('/dev/mapper/dm-uuid-mpath-%s' %
+                                             wwn)
+        mock_mpath_resize_map.side_effect = (
+            putils.ProcessExecutionError(stdout="timeout"),
+            "success")
+
+        ret_size = self.linuxscsi.extend_volume(['/dev/fake1', '/dev/fake2'],
+                                                use_multipath=True)
+        self.assertEqual(2048, ret_size)
+
+        # because we don't mock out the echo_scsi_command
+        expected_cmds = ['tee -a /sys/bus/scsi/drivers/sd/0:0:0:1/rescan',
+                         'tee -a /sys/bus/scsi/drivers/sd/1:0:0:1/rescan',
+                         'multipathd reconfigure']
+        self.assertEqual(expected_cmds, self.cmds)
+        mock_mpath_resize_map.assert_has_calls([mock.call(wwn)] * 2)
+
+    @mock.patch('time.sleep')
+    @mock.patch('time.time')
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_multipath_resize_map')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device_path')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
+    @mock.patch('os_brick.utils.get_device_size')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
+    def test_extend_volume_with_mpath_timeout(self, mock_device_info,
+                                              mock_device_size,
+                                              mock_scsi_wwn,
+                                              mock_find_mpath_path,
+                                              mock_mpath_resize_map,
+                                              mock_currtime,
+                                              mock_sleep):
+        """Test extending a volume where there is a multipath device fail."""
+        mock_device_info.side_effect = [{'host': host,
+                                         'channel': '0',
+                                         'id': '0',
+                                         'lun': '1'} for host in ['0', '1']]
+
+        mock_device_size.side_effect = [1024, 2048, 1024, 2048, 1024, 2048]
+        wwn = '1234567890123456'
+        mock_scsi_wwn.return_value = wwn
+        mock_find_mpath_path.return_value = ('/dev/mapper/dm-uuid-mpath-%s' %
+                                             wwn)
+
+        # time.time is used to check if our own timeout has been exceeded,
+        # which is why it has to be mocked.
+        fake_time = 0
+
+        def get_fake_time():
+            nonlocal fake_time
+            fake_time += 10
+            return fake_time
+
+        mock_currtime.side_effect = get_fake_time
+        # We're testing the scenario in which the multipath resize map
+        # call times out indefinitely.
+        mock_mpath_resize_map.side_effect = putils.ProcessExecutionError(
+            stdout="timeout")
+
+        self.assertRaises(
+            putils.ProcessExecutionError,
+            self.linuxscsi.extend_volume,
+            ['/dev/fake1', '/dev/fake2'],
+            use_multipath=True)
+
     def test_process_lun_id_list(self):
         lun_list = [2, 255, 88, 370, 5, 256]
         result = self.linuxscsi.process_lun_id(lun_list)
@@ -832,20 +980,59 @@ loop0                                     0"""
         self.assertEqual(expected, result)
 
     @mock.patch('os_brick.privileged.rootwrap.execute', return_value=('', ''))
-    def test_is_multipath_running_default_executor(self, mock_exec):
+    def test_is_multipath_running(self, mock_exec):
         res = linuxscsi.LinuxSCSI.is_multipath_running(False, None, mock_exec)
         self.assertTrue(res)
         mock_exec.assert_called_once_with(
             'multipathd', 'show', 'status', run_as_root=True, root_helper=None)
 
+    @mock.patch.object(linuxscsi, 'LOG')
     @mock.patch('os_brick.privileged.rootwrap.execute')
-    def test_is_multipath_running_failure_exit_code_0(self, mock_exec):
+    def test_is_multipath_running_failure(
+        self, mock_exec, mock_log
+    ):
+        mock_exec.side_effect = putils.ProcessExecutionError()
+        self.assertRaises(putils.ProcessExecutionError,
+                          linuxscsi.LinuxSCSI.is_multipath_running,
+                          True, None, mock_exec)
+        mock_log.error.assert_called_once()
+
+    @mock.patch.object(linuxscsi, 'LOG')
+    @mock.patch('os_brick.privileged.rootwrap.execute')
+    def test_is_multipath_running_failure_exit_code_0(
+        self, mock_exec, mock_log
+    ):
         mock_exec.return_value = ('error receiving packet', '')
         self.assertRaises(putils.ProcessExecutionError,
                           linuxscsi.LinuxSCSI.is_multipath_running,
                           True, None, mock_exec)
         mock_exec.assert_called_once_with(
             'multipathd', 'show', 'status', run_as_root=True, root_helper=None)
+        mock_log.error.assert_called_once()
+
+    @mock.patch.object(linuxscsi, 'LOG')
+    @mock.patch('os_brick.privileged.rootwrap.execute')
+    def test_is_multipath_running_failure_not_enforcing_multipath(
+        self, mock_exec, mock_log
+    ):
+        mock_exec.side_effect = putils.ProcessExecutionError()
+        res = linuxscsi.LinuxSCSI.is_multipath_running(False, None, mock_exec)
+        mock_exec.assert_called_once_with(
+            'multipathd', 'show', 'status', run_as_root=True, root_helper=None)
+        self.assertFalse(res)
+        mock_log.error.assert_not_called()
+
+    @mock.patch.object(linuxscsi, 'LOG')
+    @mock.patch('os_brick.privileged.rootwrap.execute')
+    def test_is_multipath_running_failure_not_enforcing_exit_code_0(
+        self, mock_exec, mock_log
+    ):
+        mock_exec.return_value = ('error receiving packet', '')
+        res = linuxscsi.LinuxSCSI.is_multipath_running(False, None, mock_exec)
+        mock_exec.assert_called_once_with(
+            'multipathd', 'show', 'status', run_as_root=True, root_helper=None)
+        self.assertFalse(res)
+        mock_log.error.assert_not_called()
 
     def test_get_device_info(self):
         ret = "[1:1:0:0] disk Vendor Array 0100 /dev/adevice\n"
@@ -860,7 +1047,7 @@ loop0                                     0"""
                                     'id': '0',
                                     'lun': '0'})
 
-    @mock.patch('six.moves.builtins.open')
+    @mock.patch('builtins.open')
     def test_get_sysfs_wwn_mpath(self, open_mock):
         wwn = '3600d0230000000000e13955cc3757800'
         cm_open = open_mock.return_value.__enter__.return_value
@@ -881,7 +1068,7 @@ loop0                                     0"""
         glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
         get_wwid_mock.assert_called_once_with(mock.sentinel.device_names)
 
-    @mock.patch('six.moves.builtins.open', side_effect=Exception)
+    @mock.patch('builtins.open', side_effect=Exception)
     @mock.patch('glob.glob')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwid')
     def test_get_sysfs_wwn_mpath_exc(self, get_wwid_mock, glob_mock,
@@ -974,7 +1161,7 @@ loop0                                     0"""
               {'wwn_type': 'eui.', 'num_val': '2'},
               {'wwn_type': 'naa.', 'num_val': '3'})
     @ddt.unpack
-    @mock.patch('six.moves.builtins.open')
+    @mock.patch('builtins.open')
     def test_get_sysfs_wwid(self, open_mock, wwn_type, num_val):
         read_fail = mock.MagicMock()
         read_fail.__enter__.return_value.read.side_effect = IOError
@@ -989,7 +1176,7 @@ loop0                                     0"""
                                     mock.call('/sys/block/sdb/device/wwid'),
                                     mock.call('/sys/block/sdc/device/wwid')])
 
-    @mock.patch('six.moves.builtins.open', side_effect=IOError)
+    @mock.patch('builtins.open', side_effect=IOError)
     def test_get_sysfs_wwid_not_found(self, open_mock):
         res = self.linuxscsi.get_sysfs_wwid(['sda', 'sdb'])
         self.assertEqual('', res)
@@ -1108,38 +1295,19 @@ loop0                                     0"""
         self.linuxscsi.multipath_add_path('/dev/sda')
         self.assertEqual(['multipathd add path /dev/sda'], self.cmds)
 
-    @ddt.data({'con_props': {}, 'dev_info': {'path': mock.sentinel.path}},
-              {'con_props': None, 'dev_info': {'path': mock.sentinel.path}},
-              {'con_props': {'device_path': mock.sentinel.device_path},
-               'dev_info': {'path': mock.sentinel.path}})
-    @ddt.unpack
-    def test_get_dev_path_device_info(self, con_props, dev_info):
-        self.assertEqual(mock.sentinel.path,
-                         self.linuxscsi.get_dev_path(con_props, dev_info))
+    def test_multipath_del_path(self):
+        self.linuxscsi.multipath_del_path('/dev/sda')
+        self.assertEqual(['multipathd del path /dev/sda'], self.cmds)
 
-    @ddt.data({'con_props': {'device_path': mock.sentinel.device_path},
-               'dev_info': {'path': None}},
-              {'con_props': {'device_path': mock.sentinel.device_path},
-               'dev_info': {'path': ''}},
-              {'con_props': {'device_path': mock.sentinel.device_path},
-               'dev_info': {}},
-              {'con_props': {'device_path': mock.sentinel.device_path},
-               'dev_info': None})
-    @ddt.unpack
-    def test_get_dev_path_conn_props(self, con_props, dev_info):
-        self.assertEqual(mock.sentinel.device_path,
-                         self.linuxscsi.get_dev_path(con_props, dev_info))
-
-    @ddt.data({'con_props': {'device_path': ''}, 'dev_info': {'path': None}},
-              {'con_props': {'device_path': None}, 'dev_info': {'path': ''}},
-              {'con_props': {}, 'dev_info': {}},
-              {'con_props': {}, 'dev_info': None})
-    @ddt.unpack
-    def test_get_dev_path_no_path(self, con_props, dev_info):
-        self.assertEqual('', self.linuxscsi.get_dev_path(con_props, dev_info))
-
-    @ddt.data(('/dev/sda', '/dev/sda', True, False, None),
+    @ddt.data(('/dev/sda', '/dev/sda', False, True, None),
+              # This checks that we ignore the was_multipath parameter if it
+              # doesn't make sense (because the used path is the one we are
+              # asking about)
+              ('/dev/sda', '/dev/sda', True, True, None),
               ('/dev/sda', '', True, False, None),
+              # Check for encrypted volume
+              ('/dev/link_sda', '/dev/disk/by-path/pci-XYZ', False, True,
+               ('/dev/sda', '/dev/mapper/crypt-pci-XYZ')),
               ('/dev/link_sda', '/dev/link_sdb', False, False, ('/dev/sda',
                                                                 '/dev/sdb')),
               ('/dev/link_sda', '/dev/link2_sda', False, True, ('/dev/sda',
